/* int_cal.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char *);
extern void int_freq1( long int );
extern void int_freq2( long int );
extern void VNA_calibrate( void );
extern int save_cal( void );
extern int restore_cal( void );
//=============================================================================
// low level screen access to display and manage the calibration screen
// 
//=============================================================================
void int_cal( void )
{
	long freq1 = cal.f_start;
	long freq2 = freq1 + ((cal.f_num-1) * cal.f_step);
	int fieldnum = 0;		// local indicator of which field is adjustable
	long int encoder;
	const int fieldpos[] = {6,16,0x42,0x55};

	lcd_goto(0);
	lcd_puts("CAL 11.234MHz 12.245MHz ");
	lcd_goto(0x40);
	lcd_puts("Abort              Start");
	int_freq2(freq2);
	int_freq1(freq1);	

	POSCNT = 0;			// reset encoder counter

	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	
			fieldnum--;
			if (fieldnum < 0) fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{
			fieldnum++;
			if (fieldnum > 3) fieldnum = 3;
			lcd_goto(fieldpos[fieldnum]); 	
			while(but2 > 0); 		//wait for button release
		}
		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );			// wait for button release
			if ( fieldnum == 2) break;	// abort calibration
			if ( fieldnum ==3 )			// start cal
			{
				if (freq1 < freq2)
				{
					cal.f_start = freq1;
					cal.f_num = CALPTS;
					cal.f_step = (freq2-freq1)/(cal.f_num-1);
					VNA_calibrate();		// collect standard data
					lcd_goto(0x40);
					lcd_puts("Writing Cal to EEPROM...");
					timer_test = 10000;		// 1 sec delay
					while( timer_test );
					save_cal();				// write cal to EEPROM
					restore_cal();			// make sure getting it back works
					break;
				}
				else
				{
					lcd_goto(0x40);
					lcd_puts("Error: F1 must be < F2  ");
					timer_test = 30000;		// 3 sec delay
					while( timer_test );
					lcd_goto(0x40);
					lcd_puts("Abort              Start");
					fieldnum = 0;		// put cursor back on 1st number
					lcd_goto(fieldpos[fieldnum]);
				}
			}
		}

		// check for relative encoder activity
		// note encoder res is halved to make it easier to set freq
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		encoder /= 2;					// reduce control sensitivity
		if (encoder)
		{
			POSCNT = 0;		// reset encoder
			if ( fieldnum == 0 )
			{
				freq1 += encoder * 50000L;
				if ( freq1 < MINFREQ ) freq1 = MINFREQ;
				if ( freq1 > freq2 ) freq1 = freq2;
				int_freq1(freq1);
			}
			if ( fieldnum == 1 )
			{
				freq2 += encoder * 50000L;
				if ( freq2 < freq1 ) freq2 = freq1;
				if ( freq2 > MAXFREQ ) freq2 = MAXFREQ;
				int_freq2(freq2);
			}
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			//TODO read hardware to display voltages
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
		}
	}
}
