/* int_cfg.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern int save_cal( void );
extern void set_pwm( unsigned int );

//=============================================================================
// low level screen access to display and manage the config screen
// 
//=============================================================================
void int_cfg( void )
{
	static	int fieldnum = 0;		// local indicator of which field is adjustable
	long int encoder;
	const int fieldpos[] = {0x40,0x4b,0x52}; // screen posns for cursor
	int samplerate;
	char *rates[] = {"fast","slow"};

	//TODO read freq, contrast and sample rate from nonvolatile memory
	// and init local vars
	samplerate = cal.samplerate * 25;

	lcd_goto(0);
	lcd_puts("  Osc:    Contr: Sample:");
	lcd_goto(0x40);
	lcd_puts("                        ");

	lcd_goto(fieldpos[0]);
	sprintf(buff30,"%9ld",cal.refclk);
	lcd_puts(buff30);

	lcd_goto(fieldpos[1]);
	sprintf(buff30,"%04d",cal.contrast);
	lcd_puts(buff30);

	lcd_goto(fieldpos[2]);
	lcd_puts(rates[cal.samplerate]);

	// set def field
	lcd_goto(fieldpos[fieldnum]);

	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	
			fieldnum--;
			if ( fieldnum < 0 )
				fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{
			fieldnum++;
			if ( fieldnum > 2 )
				fieldnum = 2;
			lcd_goto(fieldpos[fieldnum]);
			while(but2 > 0); 		//wait for button release
		}

   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			// write freq, contrast and sample rate to 
			// nonvolatile memory from  local vars
			cal.samplerate = samplerate / 25;
			save_cal();
			break;
		}
		// check for relative encoder activity
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		if (encoder)
		{
			POSCNT = 0;		// reset encoder
			switch (fieldnum)
			{
			case 0:		// vna ref osc freq (around 150mhz)

				cal.refclk += (encoder * 15);	// 15hz steps = 1hz at 10mhz
				// I am assuming user has used a ref oscillator between these
				// ranges.... if too far out, then a recompile is required
				if ( cal.refclk < 140000000L ) cal.refclk = 140000000L;
				if ( cal.refclk > 180000000L ) cal.refclk = 180000000L;

				// reset vna freq to 10mhz using new ref osc freq
				// this will let one zero beat WWV or 10mhz std osc
				sprintf(buff30,"%09ld",cal.refclk);
				lcd_puts(buff30);
				AD9851_SetFreq( 10000000L, 0L );
				break;
			case 1:		// display contrast adjustment 0..1023
				cal.contrast += encoder;
				if ( cal.contrast < 0 ) cal.contrast = 0;
				if ( cal.contrast > 1023 ) cal.contrast = 1023;
				set_pwm(cal.contrast);		// adjust voltage to lcd
				sprintf(buff30,"%04d",cal.contrast);
				lcd_puts(buff30);
				break;
			case 2:		// analog to digital sample rate
				// to reduce sensitivity... it takes 10 enc pulses to bump to next speed
				samplerate += encoder;
				if (samplerate < 0) samplerate = 0;
				if (samplerate > 25) samplerate = 25;
				lcd_puts(rates[samplerate/25]);
				break;
			}
			// restore cursor to begiining of active field
			lcd_goto(fieldpos[fieldnum]);
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			//TODO read hardware to display voltages
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
		}
	}
}
