/* int_main.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

// to conserve stack space define a general use char buffer as global
char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void VNA_Open( void );
extern int restore_cal( void );
extern void int_run( void );
extern void int_cal( void );
extern void int_cfg( void );
extern void int_tst( void );
extern void int_rmt( void );
extern void int_sdb( void );
extern void int_batt( void );
extern int calc_cksum(int sizew, int *adr);

//=============================================================================
// Stuff that may need to change for your hardware environment
//=============================================================================

//=============================================================================
// local defines.... you should not have to mess with these
//=============================================================================
#define	TRUE	(1)
#define	FALSE	(0)	

//=============================================================================
// local global vars used by this program
//=============================================================================

//=============================================================================
// low level screen access to display and manage the main screen
// 
//=============================================================================
void int_main( void )
{
	static int fieldnum = 0;		// local indicator of which field is adjustable
	timer_led = 10000;				// 1.0 seconds
	int cs;

	STATUS_LED = 1;					// led off

	lcd_goto(0);
	lcd_puts("VNA-dsp by VE7IT  0.000v");
	lcd_goto(0x40);
	lcd_puts("Run Cal Cfg Tst Rmt Sdb ");
	lcd_goto(fieldnum * 4 + 0x41);

	// size of cal structure must be <= 1024 bytes
	// to fit in eeprom
	printf("calsize=%04d bytes.\r\n",sizeof(cal));

	// Open the VNA.
	VNA_Open();

	// restore config from eeprom
	// Read array named "calEE" from DataEEPROM and place 
	// the result into array in RAM named, "cal" 
	restore_cal();
	cs = calc_cksum(sizeof(cal)/sizeof(int),(int*)&cal);
	if ( cs )
	{
		// opps, no valid calibration detected
		// assume we are starting from a new box
		cal.refclk = 149999800L;
		cal.contrast = 0;
		cal.samplerate = 0;
		cal.f_num = CALPTS;
		cal.f_start = CALSTART;
		cal.f_step = CALSTEP;

		lcd_goto(0x40);
		lcd_puts("!!Please Calibrate Me!! ");
		printf("No valid calibration found in EEPROM\r\n");
		fieldnum = 1;
	}
		
	while (1)
	{
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	
			fieldnum--;
			if ( fieldnum < 0 )
				fieldnum = 0;
			lcd_goto(fieldnum * 4 + 0x41);
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{
			fieldnum++;
			if ( fieldnum > 5 )
				fieldnum = 5;
			lcd_goto(fieldnum * 4 + 0x41);
			while(but2 > 0); 		//wait for button release
		}

   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			switch(fieldnum)
			{
			case 0:
				int_run();
				break;
			case 1:
				int_cal();
				break;
			case 2:
				int_cfg();
				break;
			case 3:
				int_tst();
				break;
			case 4:
				int_rmt();
				break;
			case 5:
				// serial debug... dump data to serial port for drift analysis
				int_sdb();
				break;

			}
			// after we return from a mode, redisplay main screen
			// using last cursor position
			lcd_goto(0);
			lcd_puts("VNA-dsp by VE7IT  0.000v");
			lcd_goto(0x40);
			lcd_puts("Run Cal Cfg Tst Rmt Sdb ");
			lcd_goto(fieldnum * 4 + 0x41);
		}
		// led outside world know we are alive (1hz flash)
		// and update battery voltage display
		if ( timer_led == 0 )
		{
			timer_led = 5000;
			STATUS_LED = ~STATUS_LED;
			int_batt();			// update battery voltage
			lcd_goto(fieldnum * 4 + 0x41);
		}
	}
}
