/* int_rmt.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 * for some reason, this file pukes the compiler when using -O2,3 or s
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern int VNA_GetReading( long int freq, __complex__ double *pResult );
extern int VNA_ComputeComplexImpedance(
		__complex__ double RhoA, 
		__complex__ double Z0,
		__complex__ double *pZx );
extern int VNA_ComputeReflCoeff(
		__complex__ double RhoM, 
		__complex__ double e00, 
		__complex__ double e11, 
		__complex__ double Deltae,
		__complex__ double *pRhoA);
extern void int_cmplx( __complex__ num );
extern int VNA_interpcal( long int freq,
					__complex__ double *e00,
					__complex__ double *e11,
					__complex__ double *Deltae );


//=============================================================================
// low level screen access to display and manage the remote cpu screen
// 
//=============================================================================
void int_rmt( void )
{
	char buff[10];
	int	buff_index;				// serial interface freq string buff
	char ch;

	long int freq = 3500000L;	// current freq requested by pc
	long int fmax;
	long int serialcnt = 0;
	__complex__ double unknown;			// raw voltages
	__complex__ double Rho;				// computed reflection coeff.
	__complex__ double Zx = 0.0+0.0i;	// computed impedance
	__complex__ double Z50 = 50.0+0.0I;
	__complex__ double e00,e11,Deltae;

	fmax = cal.f_start + ((cal.f_num -1)*cal.f_step);

	lcd_goto(0);
	lcd_puts("Rmt 12.245MHz I/O=12345 ");
	lcd_goto(0x40);
	lcd_puts("Zx=                     ");

	buff_index = 0;
	buff[0] = 0;

	// clear out serial buffer
	while(U1STAbits.URXDA) 
		ch=U1RXREG;


	while (1)
	{
		// TODO interact with serial port
		//  receive a freq, compute impedance and send it back

		if (U1STAbits.URXDA)
		{
			serialcnt++;
			ch = U1RXREG & 0xFF;
			// char available from pc put it in buffer if its a number
			if ( buff_index < (sizeof(buff) - 2) && 
				ch >= '0' &&
				ch <= '9')
			{
				buff[buff_index] = ch;
				buff[buff_index + 1] = 0;
				buff_index++;
			}

			if ( ch == 0x0d || ch == 0x0a )
			{
				// end of line, see if buffer has a frequency for us
				// we should have at least a 4 digit number in the buffer
				freq = atol( buff );
				if ( freq < cal.f_start ) 
					freq = cal.f_start;
				if ( freq > fmax ) 
					freq = fmax;

				// read hardware and convert to requested display
				VNA_interpcal(freq,&e00,&e11,&Deltae);
   				VNA_GetReading( freq, &unknown );
   				VNA_ComputeReflCoeff( unknown, e00, e11, Deltae, &Rho);
   				VNA_ComputeComplexImpedance(Rho, Z50, &Zx);

				// send real and imag parts back to pc
				printf("%ld %f %f\r\n",freq,__real__ Zx,__imag__ Zx);

				// reset input stream
				buff_index = 0;
				buff[0] = 0;
			}
		}

   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			break;
		}

		// let outside world know we are alive (1hz flash)
		// and update serial status displays
		if ( timer_led == 0 )
		{
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
			sprintf(buff30,"Rmt %9sHz %08ld",buff,serialcnt);
			lcd_goto(0);
			lcd_puts(buff30);			// display current freq and cnt
			lcd_goto(0x43);				// display current impedance
			int_cmplx(Zx);
			lcd_goto(24);				// put cursor off screen
		}
	}
}
