/* int_run.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

extern char buff30[];		// global text buffer for screen i/o

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern int VNA_GetReading( long int freq, __complex__ double *pResult );
extern int VNA_ComputeComplexImpedance(
		__complex__ double RhoA, 
		__complex__ double Z0,
		__complex__ double *pZx );
extern int VNA_ComputeReflCoeff(
		__complex__ double RhoM, 
		__complex__ double e00, 
		__complex__ double e11, 
		__complex__ double Deltae,
		__complex__ double *pRhoA);
extern int VNA_ComputeSWR(__complex__ double Rho, double *pSWR);
extern int VNA_RToP (double Re, double Im, double *pMag, double *pAngle);
extern void int_SWR( double swr );
extern int VNA_RToD (double Radians, double *pDegrees);
extern int VNA_ComputeCapacitance (double ImZx, double Freq, double *pCap);
extern int VNA_ComputeInductance(double ImZx, double Freq, double *pInd);
extern int VNA_ComputeQ(__complex__ double Zx, double *pQ);
extern int VNA_ComputeRL(__complex__ double Rho, double *pRL);
extern void int_freq1( unsigned long freq );
extern int VNA_interpcal( long int freq,
					__complex__ double *e00,
					__complex__ double *e11,
					__complex__ double *Deltae );

//=============================================================================
// low level screen access to display 2nd line data on run screen
// 
//=============================================================================
void int_result( int result, long int freq )
{
	double SWR;					// computed SWR
	double Ind;					// computed inductance
	double Cap;					// computed capatitance
	double Q;					// computed Q
	double RL;					// computed return loss in dB
	double Mag,Angle;			// polar version of impedance
	double Degrees;				// angle in degrees

	// unknown reading
	__complex__ double unknown;		// raw voltages
	__complex__ double Rho;			// computed reflection coeff.
	__complex__ double Zx;			// computed impedance
	__complex__ double Z50 = 50.0+0.0I;
	__complex__ double e00,e11,Deltae;

	// read hardware and convert to requested display
	VNA_interpcal(freq,&e00,&e11,&Deltae);
    VNA_GetReading( freq, &unknown );
    VNA_ComputeReflCoeff(unknown,e00,e11,Deltae,&Rho);
    VNA_ComputeComplexImpedance(Rho, Z50, &Zx);
    VNA_ComputeSWR(Rho, &SWR);
	int_SWR( SWR );		// display swr
	lcd_goto(0x40);
	lcd_puts("                        ");
	lcd_goto(0x40);

	switch (result)
	{
	case 1:
		// 1   Zx = +0000.00@39.51 deg
		VNA_RToP(__real__ Zx, __imag__ Zx, &Mag, &Angle);
		VNA_RToD(Angle, &Degrees);
		sprintf(buff30,"Zx = %6.2f @%6.2fdeg ",Mag,Degrees);
		lcd_puts(buff30);
		break;
	case 2:
		// 2   Rho= +0000.00 +0000.00i
		sprintf(buff30,"Rho = %7.4f %+7.4fi",__real__ Rho,__imag__ Rho);
		lcd_puts(buff30);
		break;
	case 3:
		// 3   Cx = 00000.0 pF 
		VNA_ComputeCapacitance(__imag__ Zx, (double)freq, &Cap);
		sprintf(buff30,"Cx = %7.2f pF        ",Cap*1e12);
		lcd_puts(buff30);
		break;
	case 4:
		// 4   Lx = 00000.0 uH
		VNA_ComputeInductance(__imag__ Zx, (double)freq, &Ind);
		sprintf(buff30,"Lx = %7.2f nH        ",Ind*1e9);
		lcd_puts(buff30);
		break;
	case 5:
		// 5   Qx = 0000.0
		VNA_ComputeQ( Zx, &Q);
		sprintf(buff30,"Qx = %7.2f           ",Q);
		lcd_puts(buff30);
		break;
	case 6:
		// 6   RL = 00.00 dB
		VNA_ComputeRL(Rho, &RL);
		sprintf(buff30,"RL = %7.3f dB        ",RL);
		lcd_puts(buff30);
		break;
	default:
		// case 0 and all code screwups -
		// 0   Zx = +0000.00 +0000.00i
		sprintf(buff30,"Zx = %7.2f %+7.2fi",__real__ Zx,__imag__ Zx);
		lcd_goto(0x40);
		lcd_puts(buff30);
		break;
	}
}

//=============================================================================
// low level screen access to display and manage the run screen
//	on this screen, the fieldnum (prev,next) buttons
//  determine which data is displayed on second line
// selected from:
//    012345678901234567890123
// 0   Zx = +0000.00 +0000.00i
// 1   Zx = +0000.00@39.51 deg
// 2   Rho= +0000.00 +0000.00i
// 3   Cx = 00000.0 pF 
// 4   Lx = 00000.0 uH
// 5   Qx = 0000.0
// 6   RL = 00.00 dB
//
// 
//=============================================================================
void int_run( void )
{
	static int fieldnum = 0;		// local indicator which field gets encoder adj
	static int resultnum = 0;		// 2nd line data display selection
	long freq1;
	long encoder;
	const int fieldpos[] = {5,7,8,9,0x43}; // screen posns for cursor
	// multipiers used for frequency selection
	const long int freqscale[] = {1000000L,100000L,10000L,1000L};

	long int minf = cal.f_start;
	long int maxf = cal.f_start + (cal.f_num-1) * cal.f_step;
	freq1 = (maxf + minf) / 2L;		// initial freq is middle of cal range

//	printf("min = %ld max=%ld\r\n", minf,maxf);

	lcd_goto(0);
	lcd_puts("RUN 14.234MHz  SWR 12.25");
	int_freq1(freq1);				// display real freq
	AD9851_SetFreq( freq1, 0 );		// set VNA to new freq

	int_result(resultnum,freq1);	// real second line display
	lcd_goto(fieldpos[fieldnum]);	// set cursor properly

	while (1)
	{
		// fields   0       1         2       3        4
		// fields: mhz, 100's khz, 10's khz, khz, 2nd line data
		// read buttons to see if we need to do something
		// previous button
		if ( but1 > 0 )
		{	
			fieldnum--;
			if ( fieldnum < 0 )
				fieldnum = 0;
			lcd_goto(fieldpos[fieldnum]);			// highlight freq variable
			while(but1 > 0);		// wait for button release
		}
		// next button
		if ( but2 > 0 )
		{
			fieldnum++;
			if ( fieldnum > 4)
				fieldnum = 4;
			lcd_goto(fieldpos[fieldnum]);			// highlight 2nd line data
			while(but2 > 0); 		//wait for button release
		}

   		// enter button	- exit to main menu
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			break;
		}

		// check for relative encoder activity
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention
		encoder /= 8;					// shift not same as signed divide
		if (encoder)
		{
			// get here everytime encoder changes with magnitude of change in encoder
			POSCNT = 0;		// reset encoder
			if ( fieldnum == 4 )
			{
				// cursor is on second line... play with type of results displayed
				resultnum += encoder;
				if (resultnum < 0)
					resultnum = 0;
				if (resultnum > 6)
					resultnum = 6;
				int_result(resultnum,freq1);		// update second line
				lcd_goto(fieldpos[fieldnum]);
			}
			else
			{
				freq1 += encoder * freqscale[fieldnum];
				if ( freq1 < minf ) freq1 = minf;
				if ( freq1 > maxf ) freq1 = maxf;
				AD9851_SetFreq( freq1, 0 );			// set VNA to new freq
				int_freq1(freq1);
				lcd_goto(fieldpos[fieldnum]);
				// readings are updated every 1/2 second below..
			}
			// to minimize lag, only update readings below
			// if the user has stopped twiddling encoder
			// setting timer will hold up calcs for 1/4 second
			timer_led = 2500;
		}			
		// let outside world know we are alive (1hz flash)
		if ( timer_led == 0 )
		{
			//TODO read hardware to display voltages
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;
			int_result(resultnum,freq1);		// update second line
			lcd_goto(fieldpos[fieldnum]);
		}
	}
}
