/* int_sdb.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 * for some reason, this file pukes the compiler when using -O2,3 or s
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

// to conserve stack space define a general use char buffer as global
extern char buff30[30];

extern void lcd_goto( int );
extern void lcd_puts( char * );
extern int VNA_GetReading( long int freq, __complex__ double *pResult );
extern int VNA_ComputeComplexImpedance(
		__complex__ double RhoA, 
		__complex__ double Z0,
		__complex__ double *pZx );
extern int VNA_ComputeReflCoeff(
		__complex__ double RhoM, 
		__complex__ double e00, 
		__complex__ double e11, 
		__complex__ double Deltae,
		__complex__ double *pRhoA);
extern void int_cmplx( __complex__ num );
extern int VNA_interpcal( long int freq,
					__complex__ double *e00,
					__complex__ double *e11,
					__complex__ double *Deltae );

extern unsigned int ADC10_read(void); 
extern void AD9851_Reset( void );

//=============================================================================
// low level screen access to display and manage the serial debug screen
// This mode just takes continous readings and sends them out the serial port
// This was used to characterize drift issues on the VNA board
//=============================================================================
void int_sdb( void )
{
	long int freq;				// current freq 
	long int fmax;
	unsigned long int serialcnt = 0;
	__complex__ double unknown;			// raw voltages
	__complex__ double Rho;				// computed reflection coeff.
	__complex__ double Zx = 0.0+0.0i;	// computed impedance
	__complex__ double Z50 = 50.0+0.0I;
	__complex__ double e00,e11,Deltae;
	double battery;
	unsigned short ubat;


	fmax = cal.f_start + ((cal.f_num -1)*cal.f_step);
	// This test occurs at a fixed freq in the middle of the calibration range
	freq = ( fmax + cal.f_start ) / 2L;
 
	lcd_goto(0);
	lcd_puts("Sdb 12.245MHz I/O=12345 ");
	lcd_goto(0x40);
	lcd_puts("Zx=                     ");

	while (1)
	{
   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			break;
		}

		// let outside world know we are alive (1hz flash)
		// and update serial status displays
		if ( timer_led == 0 )
		{
			serialcnt++;
			// since we are trying to get reliable powerup syncronization
			// of the DDS chips, re reset them every 5 times to see if the
			// readings stay consistant. There is some tricky issue that after
			// a power up, we are not reading the same voltages from the bridge
			// making stored calibrations invalid. This test should show if
			// a reset of the DDS chips messes things up.
			if ( (serialcnt % 5) == 0 )
			{
				AD9851_Reset();
				printf("Reset DDS chips\r\n");
			}
			// read hardware and convert to requested display
			VNA_interpcal(freq,&e00,&e11,&Deltae);
			VNA_GetReading( freq, &unknown );
			VNA_ComputeReflCoeff( unknown, e00, e11, Deltae, &Rho);
			VNA_ComputeComplexImpedance(Rho, Z50, &Zx);
			ubat = ADC10_read();
			battery = (double)ubat * BAT_SCALE;

			// send a line of numbers back to pc for analysis
			printf("%08lu %6.3f %08ld %8.6f %8.6f %8.5f %8.5f %8.3f %8.3f\r\n",
					serialcnt, battery, freq, __real__ unknown, __imag__ unknown, 
					__real__ Rho, __imag__ Rho,__real__ Zx, __imag__ Zx);

			timer_led = 10000;			// 1000ms to next update
			STATUS_LED = ~STATUS_LED;
			sprintf(buff30,"Sdb %08ldHz %08ld",freq,serialcnt);
			lcd_goto(0);
			lcd_puts(buff30);			// display current freq and cnt
			lcd_goto(0x43);				// display current impedance
			int_cmplx(Zx);
			lcd_goto(24);				// put cursor off screen
		}
	}
}
