/* int_tst.c  
 * dsPic30f4011 driver program for the N2PK Vector Network Analyzer
 * This file contains code for the operators interface.
 *
 *  Copyright (C) 2006 By Lawrence Glaister VE7IT
 *             (ve7it@shaw.ca)
 *
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Compile using: mplab ide
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>		// for PI etc
#include "vna.h"

extern volatile int but1;				// -DEBOUNCE to +DEBOUNCE
extern volatile int but2;
extern volatile int but3;

extern volatile unsigned short int timer_test;
extern volatile unsigned short timer_led;
extern void AD9851_SetFreq( unsigned long int hz, unsigned long int phase );
extern struct CAL cal;
extern struct CAL _EEDATA(32) calEE;

// to conserve stack space define a general use char buffer as global
extern char buff30[30];
extern void lcd_goto( int );
extern void lcd_puts( char * );
extern void int_freq1( unsigned long freq );
extern int VNA_GetReading( long int freq, __complex__ double *pResult );
extern void int_lowv( double num );
extern void int_batt( void );

//=============================================================================
// low level screen access to display and manage the test screen
// 
//=============================================================================
void int_tst( void )
{
	long int freq=10000000L;	// start on wwv freq at entry to test mode
	long int encoder;
	__complex__ double unknown;		// raw voltages

	lcd_goto(0);
	lcd_puts("TST                     ");
	lcd_goto(0x40);
	lcd_puts("V0=         V90=        ");
	int_batt();		// display battery voltage
	POSCNT = 0;		// reset encoder

	while (1)
	{
   		// enter button
		if ( but3 > 0 )
		{
			while( but3 > 0 );		// wait for button release
			break;
		}

		// check for relative encoder activity
		// note encoder res is halved to make it easier to set freq
		encoder = (long)POSCNT << 16;
		encoder = encoder >> 16;		// crude sign extention

		// 256 * 10khz / rev = 2.56mhz/rev
		if (encoder)
		{
			POSCNT = 0;		// reset encoder
			freq += encoder * 10000L;	// 10khz steps
			if ( freq < MINFREQ ) freq = MINFREQ;
			if ( freq > MAXFREQ ) freq = MAXFREQ;
			int_freq1(freq);
	    	AD9851_SetFreq( freq, 0 );
			// hold off readings for 0.3 second
			timer_led = 3000;
		}			

		// let outside world know we are alive (1hz flash)
		// and update hardware status displays
		if ( timer_led == 0 )
		{
			timer_led = 5000;			// 500ms to next update
			STATUS_LED = ~STATUS_LED;

			// read hardware and display voltages
		    VNA_GetReading( freq, &unknown );

			int_freq1(freq);			// display current frequency
			lcd_goto(0x43);
			int_lowv(__real__ unknown);	// display current V0
			lcd_goto(0x50);				// display current V90
			int_lowv(__imag__ unknown);
			lcd_goto(24);				// put cursor off screen
			int_batt();
		}
	}
}
