//---------------------------------------------------------------------
//	File:		timer1.c
//
//	Written By:	Lawrence Glaister VE7IT
//
// Purpose: routines to setup and use timer 1
//      
// 
//---------------------------------------------------------------------
//
// Revision History
//
// Nov 5 2005 -- first version 
//---------------------------------------------------------------------- 
#include "vna.h"

volatile unsigned short int timer_led;
volatile unsigned short int timer_test;
volatile unsigned short int timer_a2d;
volatile unsigned short int timer_lcd;
volatile unsigned short int timer_vna;

volatile int but1 = 0;				// -DEBOUNCE to +DEBOUNCE
volatile int but2 = 0;
volatile int but3 = 0;

/*********************************************************************
  Function:        void __attribute__((__interrupt__)) _T1Interrupt (void)

  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        This isr is used to periodically do certain tasks
          		   Basic rate is set to 10000intr/sec in the init function

********************************************************************/

void __attribute__((__interrupt__)) _T1Interrupt (void)
{


	IFS0bits.T1IF = 0;

	// this block of timers is used in the software for delays
	if ( timer_led > 0 )  --timer_led;
    if ( timer_test > 0 ) --timer_test;
	if ( timer_a2d > 0 )  --timer_a2d;
	if ( timer_lcd > 0 )  --timer_lcd;
	if ( timer_vna > 0 )  --timer_vna;

	// button inputs are debounced here
	// if butx var is +ve then button is pressed
	if ( _RD1  )				// "prev" button open(+5)
	{
		if ( but1 > -DEBOUNCE )		
			but1--;
	}
	else						// button closed(gnd)
	{
		if ( but1 < DEBOUNCE )
			but1++;
	}
	if ( _RD2  )			// "next" button open(+5)
	{
		if ( but2 > -DEBOUNCE )		
			but2--;
	}
	else						// button closed(gnd)
	{
		if ( but2 < DEBOUNCE )
			but2++;
	}
	if ( _RD0  )				// "enter" button open(+5)
	{
		if ( but3 > -DEBOUNCE )		
			but3--;
	}
	else						// button closed(gnd)
	{
		if ( but3 < DEBOUNCE )
			but3++;
	}
	return;
}


/*********************************************************************
  Function:        void setupTMR1(void)

  PreCondition:    None.
 
  Input:           None.

  Output:          None.

  Side Effects:    None.

  Overview:        Initialization of timer 1 as a periodic interrupt 
                   each 0.1 ms (10khz)

  Note:            None.
********************************************************************/

void setup_TMR1(void)
{
	T1CON = 0x0020;			// internal Tcy/64 clock
	TMR1 = 0;
#define PR1PRESET  (FCY/64/10000)
	PR1 = PR1PRESET;		// 0.1 ms interrupts (100us)
	T1CONbits.TON = 1;		// turn on timer 1 
	return;
}




