//---------------------------------------------------------------------
//	File:		vna.h
//
//	Written By:	Lawrence Glaister VE7IT
//
// Purpose: Various header info for the project
//      
// 
//---------------------------------------------------------------------
//
// Revision History
// March 11 2006 --   formatted into multi file project
//---------------------------------------------------------------------- 
// define which chip we are using (peripherals change)
#include <p30f4011.h>

// the following version string is printed on powerup (also in every object module)
#define CPWRT  "\r\nVNA-dsp by L.Glaister\r\n"
#define VERSION "(c)19 May 06 by VE7IT"

//#define FCY  16000000       // 16 MIPS ==> 8mhz rc osc * PLL8 /4
// this number was tweaked by hand until serial port baud was correct
#define FCY  14620000       // 14.5 MIPS ==> 7.3mhz rc osc * PLL8 /4

// define some i/o bits for the various modules
#define STATUS_LED 	_RF1
#define ENC_LED    	_RD3

// for some reason PI may not be defined in math.h on some systems
#ifndef M_PI
#define M_PI (3.14159265358979323846)
#endif

#define	TRUE	(1)
#define	FALSE	(0)	

// measured scale for battery voltage input on AN0 volts/count
#define BAT_SCALE ( 0.01445 )	/*volts/cnt*/

#define DEBOUNCE (50)		/* button debounce = 5ms */

// define the VNA frequency tuning range
#define MINFREQ (50000L)
#define MAXFREQ (60000000L)


// this is the data we fill in during calibration
// number of points (freqs) in calibration table
// range of vna is 50khz to 60 mhz
#define CALPTS 30
#define CALSTART MINFREQ      
#define CALSTEP (( MAXFREQ - CALSTART ) / (CALPTS-1))

struct CAL_IN
{
	// measured volatges from test loads
	__complex__ double M1[CALPTS];      // voltage from open load
    __complex__ double M2[CALPTS];      // voltage from shorted load
    __complex__ double M3[CALPTS];      // voltage from 50 ohm standard
};

// this is the stuff we need to save in eeprom (same size as above)
struct CAL_OUT
{
	// calibration terms calced at time of calibration
    __complex__ double e00[CALPTS];     // computed error term
    __complex__ double e11[CALPTS];     // computed error term
    __complex__ double Deltae[CALPTS];  // computed error term
};

// this is the actual defn used in ram and written as a block to eeprom
// This has to be smaller than 1024 bytes to fit in eeprom
struct CAL
{
	int cksum;							// used to determine of config is valid
	long int refclk;					// 150mhz clk
	int contrast;						// displaycontrast adjustment 0..1023
	int samplerate;						// a sample rate index
	int spare1;
	int spare2;
	int spare3;
	char spare4[16];

	// start of 2nd ROW
    long int f_num, f_start, f_step;	// freq in hz of calibration points
	union
	{
		struct CAL_IN cal_in;
		struct CAL_OUT cal_out;
	};
};

